@@ -88,9 +88,11 @@ typedef struct
   gchar *username;
 } AutodiscoverAuthData;
 
-static void
-ews_client_autodiscover_data_free (AutodiscoverData *data)
+static gboolean
+ews_client_autodiscover_data_free (gpointer user_data)
 {
+  AutodiscoverData *data = user_data;
+
   if (data->cancellable_id > 0)
     {
       g_cancellable_disconnect (data->cancellable, data->cancellable_id);
@@ -102,6 +104,8 @@ ews_client_autodiscover_data_free (AutodiscoverData *data)
   g_object_unref (data->res);
   g_object_unref (data->session);
   g_slice_free (AutodiscoverData, data);
+
+  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -179,7 +183,7 @@ ews_client_autodiscover_response_cb (SoupSession *session, SoupMessage *msg, gpo
   xmlNode *node;
 
   status = msg->status_code;
-  if (status == SOUP_STATUS_CANCELLED)
+  if (status == SOUP_STATUS_NONE)
     return;
 
   error = NULL;
@@ -196,7 +200,9 @@ ews_client_autodiscover_response_cb (SoupSession *session, SoupMessage *msg, gpo
 
   data->msgs[idx] = NULL;
 
-  if (status != SOUP_STATUS_OK)
+  if (status == SOUP_STATUS_CANCELLED)
+    goto out;
+  else if (status != SOUP_STATUS_OK)
     {
       g_set_error (&error,
                    GOA_ERROR,
@@ -284,13 +290,14 @@ ews_client_autodiscover_response_cb (SoupSession *session, SoupMessage *msg, gpo
            * message, the callback (ie. this function) will be invoked before
            * soup_session_cancel_message returns.
            */
-          soup_session_cancel_message (data->session, data->msgs[idx], SOUP_STATUS_CANCELLED);
+          soup_session_cancel_message (data->session, data->msgs[idx], SOUP_STATUS_NONE);
           data->msgs[idx] = NULL;
         }
     }
 
  out:
-  if (error != NULL)
+  /* error == NULL, if we are being aborted by the GCancellable */
+  if (!op_res)
     {
       for (idx = 0; idx < size; idx++)
         {
@@ -303,13 +310,13 @@ ews_client_autodiscover_response_cb (SoupSession *session, SoupMessage *msg, gpo
               return;
             }
         }
-      g_simple_async_result_set_from_error (data->res, error);
+      if (error != NULL)
+        g_simple_async_result_take_error (data->res, error);
     }
-  else
-    g_simple_async_result_set_op_res_gboolean (data->res, op_res);
 
+  g_simple_async_result_set_op_res_gboolean (data->res, op_res);
   g_simple_async_result_complete_in_idle (data->res);
-  ews_client_autodiscover_data_free (data);
+  g_idle_add (ews_client_autodiscover_data_free, data);
 }
 
 static xmlDoc *
@@ -445,6 +452,7 @@ goa_ews_client_autodiscover (GoaEwsClient        *client,
                                                     G_CALLBACK (ews_client_autodiscover_cancelled_cb),
                                                     data,
                                                     NULL);
+      g_simple_async_result_set_check_cancellable (data->res, data->cancellable);
     }
 
   auth = g_slice_new0 (AutodiscoverAuthData);
