@@ -28,7 +28,6 @@
 #include "goaprovider.h"
 #include "goaexchangeprovider.h"
 #include "goaeditablelabel.h"
-#include "goaspinnerbutton.h"
 #include "goautils.h"
 
 /**
@@ -383,12 +382,15 @@ add_entry (GtkWidget     *grid1,
 
 typedef struct
 {
+  GCancellable *cancellable;
+
   GtkDialog *dialog;
   GMainLoop *loop;
 
   GtkWidget *cluebar;
   GtkWidget *cluebar_label;
-  GtkWidget *spinner_button;
+  GtkWidget *connect_button;
+  GtkWidget *progress_grid;
 
   GtkWidget *email_address;
   GtkWidget *password;
@@ -469,10 +471,13 @@ create_account_details_ui (GoaProvider    *provider,
                            gboolean        new_account,
                            AddAccountData *data)
 {
+  GtkWidget *action_area;
   GtkWidget *grid0;
   GtkWidget *grid1;
   GtkWidget *grid2;
   GtkWidget *hbox;
+  GtkWidget *label;
+  GtkWidget *spinner;
   gint width;
 
   goa_utils_set_dialog_title (provider, dialog, new_account);
@@ -543,11 +548,30 @@ create_account_details_ui (GoaProvider    *provider,
   g_signal_connect (data->email_address, "changed", G_CALLBACK (on_email_address_or_password_changed), data);
   g_signal_connect (data->password, "changed", G_CALLBACK (on_email_address_or_password_changed), data);
 
-  data->spinner_button = goa_spinner_button_new_from_stock (GTK_STOCK_CONNECT);
-  gtk_dialog_add_action_widget (data->dialog, data->spinner_button, GTK_RESPONSE_OK);
+  data->connect_button = gtk_dialog_add_button (data->dialog, GTK_STOCK_CONNECT, GTK_RESPONSE_OK);
   gtk_dialog_set_default_response (data->dialog, GTK_RESPONSE_OK);
   gtk_dialog_set_response_sensitive (data->dialog, GTK_RESPONSE_OK, FALSE);
 
+  action_area = gtk_dialog_get_action_area (data->dialog);
+
+  data->progress_grid = gtk_grid_new ();
+  gtk_widget_set_no_show_all (data->progress_grid, TRUE);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (data->progress_grid), GTK_ORIENTATION_HORIZONTAL);
+  gtk_grid_set_column_spacing (GTK_GRID (data->progress_grid), 3);
+  gtk_box_pack_end (GTK_BOX (action_area), data->progress_grid, FALSE, FALSE, 0);
+  gtk_box_reorder_child (GTK_BOX (action_area), data->progress_grid, 0);
+  gtk_button_box_set_child_non_homogeneous (GTK_BUTTON_BOX (action_area), data->progress_grid, TRUE);
+
+  spinner = gtk_spinner_new ();
+  gtk_widget_set_size_request (spinner, 20, 20);
+  gtk_widget_show (spinner);
+  gtk_spinner_start (GTK_SPINNER (spinner));
+  gtk_container_add (GTK_CONTAINER (data->progress_grid), spinner);
+
+  label = gtk_label_new (_("Connecting…"));
+  gtk_widget_show (label);
+  gtk_container_add (GTK_CONTAINER (data->progress_grid), label);
+
   gtk_window_get_size (GTK_WINDOW (data->dialog), &width, NULL);
   gtk_widget_set_size_request (GTK_WIDGET (data->dialog), width, -1);
 }
@@ -573,7 +597,17 @@ autodiscover_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 
   goa_ews_client_autodiscover_finish (client, res, &data->error);
   g_main_loop_quit (data->loop);
-  goa_spinner_button_stop (GOA_SPINNER_BUTTON (data->spinner_button));
+  gtk_widget_set_sensitive (data->connect_button, TRUE);
+  gtk_widget_hide (data->progress_grid);
+}
+
+static void
+dialog_response_cb (GtkDialog *dialog, gint response_id, gpointer user_data)
+{
+  AddAccountData *data = user_data;
+
+  if (response_id == GTK_RESPONSE_CANCEL)
+    g_cancellable_cancel (data->cancellable);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -601,12 +635,14 @@ add_account (GoaProvider    *provider,
   ret = NULL;
 
   memset (&data, 0, sizeof (AddAccountData));
+  data.cancellable = g_cancellable_new ();
   data.loop = g_main_loop_new (NULL, FALSE);
   data.dialog = dialog;
   data.error = NULL;
 
   create_account_details_ui (provider, dialog, vbox, TRUE, &data);
   gtk_widget_show_all (GTK_WIDGET (vbox));
+  g_signal_connect (dialog, "response", G_CALLBACK (dialog_response_cb), &data);
 
   ews_client = goa_ews_client_new ();
 
@@ -637,18 +673,30 @@ add_account (GoaProvider    *provider,
                                   &data.error))
     goto out;
 
+  g_cancellable_reset (data.cancellable);
   goa_ews_client_autodiscover (ews_client,
                                email_address,
                                password,
                                username,
                                server,
-                               NULL,
+                               data.cancellable,
                                autodiscover_cb,
                                &data);
-  goa_spinner_button_start (GOA_SPINNER_BUTTON (data.spinner_button));
+  gtk_widget_set_sensitive (data.connect_button, FALSE);
+  gtk_widget_show (data.progress_grid);
   g_main_loop_run (data.loop);
 
-  if (data.error != NULL)
+  if (g_cancellable_is_cancelled (data.cancellable))
+    {
+      g_prefix_error (&data.error,
+                      _("Dialog was dismissed (%s, %d): "),
+                      g_quark_to_string (data.error->domain),
+                      data.error->code);
+      data.error->domain = GOA_ERROR;
+      data.error->code = GOA_ERROR_DIALOG_DISMISSED;
+      goto out;
+    }
+  else if (data.error != NULL)
     {
       gchar *markup;
 
@@ -660,7 +708,7 @@ add_account (GoaProvider    *provider,
       gtk_label_set_markup (GTK_LABEL (data.cluebar_label), markup);
       g_free (markup);
 
-      goa_spinner_button_set_label (GOA_SPINNER_BUTTON (data.spinner_button), _("_Try Again"));
+      gtk_button_set_label (GTK_BUTTON (data.connect_button), _("_Try Again"));
       gtk_expander_set_expanded (GTK_EXPANDER (data.expander), TRUE);
       gtk_widget_set_no_show_all (data.cluebar, FALSE);
       gtk_widget_show_all (data.cluebar);
@@ -711,6 +759,7 @@ add_account (GoaProvider    *provider,
   g_free (data.account_object_path);
   if (data.loop != NULL)
     g_main_loop_unref (data.loop);
+  g_clear_object (&data.cancellable);
   if (ews_client != NULL)
     g_object_unref (ews_client);
   return ret;
@@ -761,6 +810,7 @@ refresh_account (GoaProvider    *provider,
   gtk_box_set_spacing (GTK_BOX (vbox), 12);
 
   memset (&data, 0, sizeof (AddAccountData));
+  data.cancellable = g_cancellable_new ();
   data.loop = g_main_loop_new (NULL, FALSE);
   data.dialog = GTK_DIALOG (dialog);
   data.error = NULL;
@@ -773,6 +823,7 @@ refresh_account (GoaProvider    *provider,
   gtk_editable_set_editable (GTK_EDITABLE (data.email_address), FALSE);
 
   gtk_widget_show_all (dialog);
+  g_signal_connect (dialog, "response", G_CALLBACK (dialog_response_cb), &data);
 
   ews_client = goa_ews_client_new ();
 
@@ -780,7 +831,7 @@ refresh_account (GoaProvider    *provider,
   response = gtk_dialog_run (GTK_DIALOG (dialog));
   if (response != GTK_RESPONSE_OK)
     {
-      g_set_error (error,
+      g_set_error (&data.error,
                    GOA_ERROR,
                    GOA_ERROR_DIALOG_DISMISSED,
                    _("Dialog was dismissed"));
@@ -793,20 +844,31 @@ refresh_account (GoaProvider    *provider,
   exchange = goa_object_peek_exchange (object);
   server = goa_exchange_get_host (exchange);
 
+  g_cancellable_reset (data.cancellable);
   goa_ews_client_autodiscover (ews_client,
                                email_address,
                                password,
                                username,
                                server,
-                               NULL,
+                               data.cancellable,
                                autodiscover_cb,
                                &data);
-  goa_spinner_button_start (GOA_SPINNER_BUTTON (data.spinner_button));
+  gtk_widget_set_sensitive (data.connect_button, FALSE);
+  gtk_widget_show (data.progress_grid);
   g_main_loop_run (data.loop);
 
-  if (data.error != NULL)
+  if (g_cancellable_is_cancelled (data.cancellable))
+    {
+      g_prefix_error (&data.error,
+                      _("Dialog was dismissed (%s, %d): "),
+                      g_quark_to_string (data.error->domain),
+                      data.error->code);
+      data.error->domain = GOA_ERROR;
+      data.error->code = GOA_ERROR_DIALOG_DISMISSED;
+      goto out;
+    }
+  else if (data.error != NULL)
     {
-      GtkWidget *button;
       gchar *markup;
 
       markup = g_strdup_printf ("<b>%s:</b> %s",
@@ -817,8 +879,7 @@ refresh_account (GoaProvider    *provider,
       gtk_label_set_markup (GTK_LABEL (data.cluebar_label), markup);
       g_free (markup);
 
-      button = gtk_dialog_get_widget_for_response (data.dialog, GTK_RESPONSE_OK);
-      gtk_button_set_label (GTK_BUTTON (button), _("_Try Again"));
+      gtk_button_set_label (GTK_BUTTON (data.connect_button), _("_Try Again"));
       gtk_widget_set_no_show_all (data.cluebar, FALSE);
       gtk_widget_show_all (data.cluebar);
       goto ews_again;
@@ -832,7 +893,7 @@ refresh_account (GoaProvider    *provider,
                                                     object,
                                                     g_variant_builder_end (&builder),
                                                     NULL, /* GCancellable */
-                                                    error))
+                                                    &data.error))
     goto out;
 
   goa_account_call_ensure_credentials (account,
@@ -842,9 +903,13 @@ refresh_account (GoaProvider    *provider,
   ret = TRUE;
 
  out:
+  if (data.error != NULL)
+    g_propagate_error (error, data.error);
+
   gtk_widget_destroy (dialog);
   if (data.loop != NULL)
     g_main_loop_unref (data.loop);
+  g_clear_object (&data.cancellable);
   if (ews_client != NULL)
     g_object_unref (ews_client);
   return ret;
