@@ -80,9 +80,11 @@ typedef struct
   gchar *username;
 } CheckAuthData;
 
-static void
-http_client_check_data_free (CheckData *data)
+static gboolean
+http_client_check_data_free (gpointer user_data)
 {
+  CheckData *data = user_data;
+
   if (data->cancellable_id > 0)
     {
       g_cancellable_disconnect (data->cancellable, data->cancellable_id);
@@ -93,6 +95,8 @@ http_client_check_data_free (CheckData *data)
   g_object_unref (data->res);
   g_object_unref (data->session);
   g_slice_free (CheckData, data);
+
+  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -157,7 +161,21 @@ http_client_check_response_cb (SoupSession *session, SoupMessage *msg, gpointer
     g_simple_async_result_take_error (data->res, error);
 
   g_simple_async_result_complete_in_idle (data->res);
-  http_client_check_data_free (data);
+  g_idle_add (http_client_check_data_free, data);
+}
+
+static void
+http_client_log_printer (SoupLogger         *logger,
+                         SoupLoggerLogLevel  level,
+                         gchar               direction,
+                         const gchar        *data,
+                         gpointer            user_data)
+{
+  gchar *message;
+
+  message = g_strdup_printf ("%c %s", direction, data);
+  g_log_default_handler ("goa", G_LOG_LEVEL_DEBUG, message, NULL);
+  g_free (message);
 }
 
 void
@@ -171,6 +189,7 @@ goa_http_client_check (GoaHttpClient       *client,
 {
   CheckData *data;
   CheckAuthData *auth;
+  SoupLogger *logger;
 
   g_return_if_fail (GOA_IS_HTTP_CLIENT (client));
   g_return_if_fail (uri != NULL || uri[0] != '\0');
@@ -182,6 +201,10 @@ goa_http_client_check (GoaHttpClient       *client,
   data->res = g_simple_async_result_new (G_OBJECT (client), callback, user_data, goa_http_client_check);
   data->session = soup_session_async_new_with_options (SOUP_SESSION_USE_THREAD_CONTEXT, TRUE,
                                                        NULL);
+  logger = soup_logger_new (SOUP_LOGGER_LOG_BODY, -1);
+  soup_logger_set_printer (logger, http_client_log_printer, NULL, NULL);
+  soup_session_add_feature (data->session, SOUP_SESSION_FEATURE (logger));
+  g_object_unref (logger);
 
   data->msg = soup_message_new (SOUP_METHOD_GET, uri);
   soup_message_headers_append (data->msg->request_headers, "Connection", "close");
